#include <opencv/cv.h>
#include <opencv/highgui.h>
#include <ctype.h>
#include <iostream>
#include <vector>
#include <list>
#include <cstdio>
#include <cmath>
#include <cstdlib>

using namespace std;

class Ising {
private:
  vector <vector <int> >&  map; // +1 or -1
  int width, height;
public:
  Ising( vector <vector <int> >& m ): map(m) {
    width = map[0].size();
    height = map.size();
  }
  ~Ising()
  {
  }
  void update( float kT, float H );
  IplImage* show(int zoom);
  IplImage* showo(int zoom, int yofs, IplImage* canvas);
};


void
Ising::update( float kT, float H )
{
  int x = lrand48() % width;
  int y = lrand48() % height;
  int spin = map[x][y];
  int xi = (x+1) % width;
  int yi = (y+1) % height;
  int xd = (x + width - 1) % width;
  int yd = (y + height - 1) % height;
  int neighbors = map[xi][y] + map[x][yi] + map[xd][y] + map[x][yd];
  int e_before = -spin * neighbors;
  int e_after  = +spin * neighbors;
  float de       = e_after - e_before + spin * H;
  int accept = false;
  if ( de < 0.0 ){
      accept = true;
    }
    else if ( exp(-de / kT) > drand48() ){
    accept = true;
  }
  if ( accept ){
    map[x][y] = -map[x][y];
  }
}


IplImage* Ising::show(int zoom)
{
  IplImage* img = cvCreateImage( cvSize(width*zoom,height*zoom), IPL_DEPTH_8U, 3 );
  cvZero( img );
  for(int i=0;i<height;i++){
    for(int j=0;j<width;j++){
      if ( map[i][j] > 0 ){
	cvRectangle( img,
		     cvPoint(i*zoom,j*zoom),
		     cvPoint((i+1)*zoom-2,(j+1)*zoom-2),
		     cvScalar(255,255,255)
		     );
      }
    }
  }
  return img;
}



IplImage* Ising::showo(int zoom, int yofs, IplImage* canvas)
{
  for(int i=0;i<height;i++){
    for(int j=0;j<width;j++){
      if ( map[i][j] > 0 ){
	cvRectangle( canvas,
		     cvPoint(i*zoom+j,(j+yofs)),
		     cvPoint((i+1)*zoom-2+j,(j+yofs)),
		     cvScalar(255,255,255)
		     );
	cvRectangle( canvas,
		     cvPoint((i+1)*zoom-1+j,(j+yofs)),
		     cvPoint((i+1)*zoom-1+j,(j+yofs)),
		     cvScalar(128,128,128)
		     );
      }
    }
  }
  return canvas;
}

      


int main( int argc, char* argv[] )
{
  vector <vector <int> > map;
  int height = 100;
  int width = 100;
  map.resize(height);
  for(int i=0;i<height;i++){
    map[i].resize(width);
    for(int j=0;j<width;j++){
      map[i][j] = -1;
    }
  }
  Ising ising(map);
  int fps = 30;
  int zoom = 3;
  IplImage* canvas = cvCreateImage( cvSize(width*zoom+height,height+500), IPL_DEPTH_8U, 3 );
  cvZero( canvas );
  for(int loop=0;loop<500;loop++){
    cout << loop << endl;
    for(int i=0;i<10000;i++){
      ising.update(1.0,1.2);
    }
    ising.showo(3, loop, canvas);
    cvShowImage("Ising", canvas );
    char filename[1000];
    sprintf(filename, "ising_o.%04d.jpg", loop);
    cvSaveImage( filename, canvas );
    
    char c = cvWaitKey (10);
    if (c == '\x1b')
      break;
  }
  //cvReleaseVideoWriter( &writer );
  cvReleaseImage( &canvas );
  exit(0);
}

